# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function

import time 
import argparse

# Snap ML APIs
from snapml import MPI
from snapml import io
from snapml import LinearRegression
from snapml.metrics import mean_squared_error as snapMSE

# sklearn APIs
from sklearn.linear_model import Ridge
from sklearn.metrics import mean_squared_error
from sklearn.datasets import load_svmlight_file

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
  default=defaultPath
)
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
args = CLI.parse_args()

(comm_size, rank) = MPI.Comm_get_info()

data_x,data_y = io.load_snap_file(args.data_path + "/data/small_sparse.snap")

clf_snap = LinearRegression(device_ids=args.device_ids, use_gpu=args.use_gpu, max_iter=15)

# Training
t0 = time.time()
clf_snap.fit(data_x,data_y)
if (rank == 0):
    print("[snapml] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

# Inference
pred_vals = clf_snap.predict(data_x)

# Evaluate log-loss on test set
mse = snapMSE(data_y, pred_vals)
if (rank == 0):
    print("[snapml] Mean squared error:    %.4f" % (mse))

if args.skip_sklearn:
    if (rank == 0):
        print('Execution using scikit-learn is skipped.')
    exit()

X,y = load_svmlight_file(args.data_path + "/data/small.libsvm")

# Create a RidgeRegression with sklearn
clf = Ridge(fit_intercept=False)

# Training time
t0 = time.time()
clf.fit(X, y)

if (rank == 0):
    print("[sklearn] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

# Inference
pred_vals = clf.predict(X)

# Evaluate log-loss on test set
L2loss_sklearn = mean_squared_error(y, pred_vals)
if (rank == 0):
    print("[sklearn] MPI process %d : Mean squared error:    %.4f" % (rank, L2loss_sklearn))
